#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2016 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
   Plugin for duplicating object graphs

"""

import sys
import os
import warnings

from omero.cli import CLI, GraphControl

HELP = """Duplicate OMERO data.

Duplicate entire graphs of data based on the ID of the top-node.

Note that no object that corresponds to a file on disk will be duplicated.
In some circumstances a duplicate object may reference an original object that
does have associated binary data but a duplicated image should not be expected
to include any pixel data.

Examples:

    # Duplicate a dataset
    omero duplicate Dataset:50
    # Do the same reporting all the new duplicate objects
    omero duplicate Dataset:50 --report

    # Do a dry run of a duplicate reporting the outcome
    # if the duplicate had been run
    omero duplicate Dataset:53 --dry-run
    # Do a dry run of a duplicate, reporting all the objects
    # that would have been duplicated
    omero duplicate Dataset:53 --dry-run --report

"""


DEPRECATION_MESSAGE = (
    "This plugin is deprecated as of OMERO.py 5.8.0. Use the plugin"
    " available from https://pypi.org/project/omero-cli-duplicate/"
    " instead.")


class DuplicateControl(GraphControl):

    def main_method(self, args):
        self.ctx.err(DEPRECATION_MESSAGE, DeprecationWarning)
        super(DuplicateControl, self).main_method(args)

    def cmd_type(self):
        import omero
        import omero.all
        return omero.cmd.Duplicate

    def print_detailed_report(self, req, rsp, status):
        import omero
        if isinstance(rsp, omero.cmd.DoAllRsp):
            for response in rsp.responses:
                if isinstance(response, omero.cmd.DuplicateResponse):
                    self.print_duplicate_response(response)
        elif isinstance(rsp, omero.cmd.DuplicateResponse):
            self.print_duplicate_response(rsp)

    def print_duplicate_response(self, rsp):
        if rsp.duplicates:
            self.ctx.out("Duplicates")
            objIds = self._get_object_ids(rsp.duplicates)
            for k in objIds:
                self.ctx.out("  %s:%s" % (k, objIds[k]))

try:
    if "OMERO_DEV_PLUGINS" in os.environ:
        warnings.warn(DEPRECATION_MESSAGE, DeprecationWarning)
        register("duplicate", DuplicateControl, HELP)
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("duplicate", DuplicateControl, HELP)
        cli.invoke(sys.argv[1:])



"""
   /*
   **   Generated by blitz/resources/templates/combined.vm
   **
   **   Copyright 2007, 2008 Glencoe Software, Inc. All rights reserved.
   **   Use is subject to license terms supplied in LICENSE.txt
   **
   */
"""
try:
   unicode
except NameError:
   # Python 3: "unicode" is built-in
   unicode = str
import Ice
import IceImport
import omero
IceImport.load("omero_model_DetailsI")
IceImport.load("omero_model_Plate_ice")
from omero.rtypes import rlong
from collections import namedtuple
_omero = Ice.openModule("omero")
_omero_model = Ice.openModule("omero.model")
__name__ = "omero.model"
class PlateI(_omero_model.Plate):

      # Property Metadata
      _field_info_data = namedtuple("FieldData", ["wrapper", "nullable"])
      _field_info_type = namedtuple("FieldInfo", [
          "defaultSample",
          "columnNamingConvention",
          "rowNamingConvention",
          "wellOriginX",
          "wellOriginY",
          "rows",
          "columns",
          "status",
          "externalIdentifier",
          "screenLinks",
          "wells",
          "plateAcquisitions",
          "annotationLinks",
          "name",
          "description",
          "details",
      ])
      _field_info = _field_info_type(
          defaultSample=_field_info_data(wrapper=omero.rtypes.rint, nullable=True),
          columnNamingConvention=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          rowNamingConvention=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          wellOriginX=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          wellOriginY=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          rows=_field_info_data(wrapper=omero.rtypes.rint, nullable=True),
          columns=_field_info_data(wrapper=omero.rtypes.rint, nullable=True),
          status=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          externalIdentifier=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          screenLinks=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          wells=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          plateAcquisitions=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          annotationLinks=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          name=_field_info_data(wrapper=omero.rtypes.rstring, nullable=False),
          description=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          details=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
      )  # end _field_info
      DEFAULTSAMPLE =  "ome.model.screen.Plate_defaultSample"
      COLUMNNAMINGCONVENTION =  "ome.model.screen.Plate_columnNamingConvention"
      ROWNAMINGCONVENTION =  "ome.model.screen.Plate_rowNamingConvention"
      WELLORIGINX =  "ome.model.screen.Plate_wellOriginX"
      WELLORIGINY =  "ome.model.screen.Plate_wellOriginY"
      ROWS =  "ome.model.screen.Plate_rows"
      COLUMNS =  "ome.model.screen.Plate_columns"
      STATUS =  "ome.model.screen.Plate_status"
      EXTERNALIDENTIFIER =  "ome.model.screen.Plate_externalIdentifier"
      SCREENLINKS =  "ome.model.screen.Plate_screenLinks"
      WELLS =  "ome.model.screen.Plate_wells"
      PLATEACQUISITIONS =  "ome.model.screen.Plate_plateAcquisitions"
      ANNOTATIONLINKS =  "ome.model.screen.Plate_annotationLinks"
      NAME =  "ome.model.screen.Plate_name"
      DESCRIPTION =  "ome.model.screen.Plate_description"
      DETAILS =  "ome.model.screen.Plate_details"
      def errorIfUnloaded(self):
          if not self._loaded:
              raise _omero.UnloadedEntityException("Object unloaded:"+str(self))

      def throwNullCollectionException(self,propertyName):
          raise _omero.UnloadedEntityException(""+
          "Error updating collection:" + propertyName +"\n"+
          "Collection is currently null. This can be seen\n" +
          "by testing \""+ propertyName +"Loaded\". This implies\n"+
          "that this collection was unloaded. Please refresh this object\n"+
          "in order to update this collection.\n")

      def _toggleCollectionsLoaded(self, load):
          if load:
              self._screenLinksSeq = []
              self._screenLinksLoaded = True;
          else:
              self._screenLinksSeq = []
              self._screenLinksLoaded = False;

          if load:
              self._wellsSeq = []
              self._wellsLoaded = True;
          else:
              self._wellsSeq = []
              self._wellsLoaded = False;

          if load:
              self._plateAcquisitionsSeq = []
              self._plateAcquisitionsLoaded = True;
          else:
              self._plateAcquisitionsSeq = []
              self._plateAcquisitionsLoaded = False;

          if load:
              self._annotationLinksSeq = []
              self._annotationLinksLoaded = True;
          else:
              self._annotationLinksSeq = []
              self._annotationLinksLoaded = False;

          pass

      def __init__(self, id=None, loaded=None):
          super(PlateI, self).__init__()
          if id is not None and isinstance(id, (str, unicode)) and ":" in id:
              parts = id.split(":")
              if len(parts) != 2:
                  raise Exception("Invalid proxy string: %s", id)
              if parts[0] != self.__class__.__name__ and \
                 parts[0]+"I" != self.__class__.__name__:
                  raise Exception("Proxy class mismatch: %s<>%s" %
                  (self.__class__.__name__, parts[0]))
              self._id = rlong(parts[1])
              if loaded is None:
                  # If no loadedness was requested with
                  # a proxy string, then assume False.
                  loaded = False
          else:
              # Relying on omero.rtypes.rlong's error-handling
              self._id = rlong(id)
              if loaded is None:
                  loaded = True  # Assume true as previously
          self._loaded = loaded
          if self._loaded:
             self._details = _omero_model.DetailsI()
             self._toggleCollectionsLoaded(True)

      def unload(self, current = None):
          self._loaded = False
          self.unloadDefaultSample( )
          self.unloadColumnNamingConvention( )
          self.unloadRowNamingConvention( )
          self.unloadWellOriginX( )
          self.unloadWellOriginY( )
          self.unloadRows( )
          self.unloadColumns( )
          self.unloadStatus( )
          self.unloadExternalIdentifier( )
          self.unloadScreenLinks( )
          self.unloadWells( )
          self.unloadPlateAcquisitions( )
          self.unloadAnnotationLinks( )
          self.unloadName( )
          self.unloadDescription( )
          self.unloadDetails( )

      def isLoaded(self, current = None):
          return self._loaded
      def unloadCollections(self, current = None):
          self._toggleCollectionsLoaded( False )
      def isGlobal(self, current = None):
          return  False ;
      def isMutable(self, current = None):
          return  True ;
      def isAnnotated(self, current = None):
          return  True ;
      def isLink(self, current = None):
          return  False ;
      def shallowCopy(self, current = None):
            if not self._loaded: return self.proxy()
            copy = PlateI()
            copy._id = self._id;
            copy._version = self._version;
            copy._details = None  # Unloading for the moment.
            raise omero.ClientError("NYI")
      def proxy(self, current = None):
          if self._id is None: raise omero.ClientError("Proxies require an id")
          return PlateI( self._id.getValue(), False )

      def getDetails(self, current = None):
          self.errorIfUnloaded()
          return self._details

      def unloadDetails(self, current = None):
          self._details = None

      def getId(self, current = None):
          return self._id

      def setId(self, _id, current = None):
          self._id = _id

      def checkUnloadedProperty(self, value, loadedField):
          if value == None:
              self.__dict__[loadedField] = False
          else:
              self.__dict__[loadedField] = True

      def getVersion(self, current = None):
          self.errorIfUnloaded()
          return self._version

      def setVersion(self, version, current = None):
          self.errorIfUnloaded()
          self._version = version

      def unloadDefaultSample(self, ):
          self._defaultSampleLoaded = False
          self._defaultSample = None;

      def getDefaultSample(self, current = None):
          self.errorIfUnloaded()
          return self._defaultSample

      def setDefaultSample(self, _defaultSample, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.defaultSample.wrapper is not None:
              if _defaultSample is not None:
                  _defaultSample = self._field_info.defaultSample.wrapper(_defaultSample)
          self._defaultSample = _defaultSample
          pass

      def unloadColumnNamingConvention(self, ):
          self._columnNamingConventionLoaded = False
          self._columnNamingConvention = None;

      def getColumnNamingConvention(self, current = None):
          self.errorIfUnloaded()
          return self._columnNamingConvention

      def setColumnNamingConvention(self, _columnNamingConvention, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.columnNamingConvention.wrapper is not None:
              if _columnNamingConvention is not None:
                  _columnNamingConvention = self._field_info.columnNamingConvention.wrapper(_columnNamingConvention)
          self._columnNamingConvention = _columnNamingConvention
          pass

      def unloadRowNamingConvention(self, ):
          self._rowNamingConventionLoaded = False
          self._rowNamingConvention = None;

      def getRowNamingConvention(self, current = None):
          self.errorIfUnloaded()
          return self._rowNamingConvention

      def setRowNamingConvention(self, _rowNamingConvention, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.rowNamingConvention.wrapper is not None:
              if _rowNamingConvention is not None:
                  _rowNamingConvention = self._field_info.rowNamingConvention.wrapper(_rowNamingConvention)
          self._rowNamingConvention = _rowNamingConvention
          pass

      def unloadWellOriginX(self, ):
          self._wellOriginXLoaded = False
          self._wellOriginX = None;

      def getWellOriginX(self, current = None):
          self.errorIfUnloaded()
          return self._wellOriginX

      def setWellOriginX(self, _wellOriginX, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.wellOriginX.wrapper is not None:
              if _wellOriginX is not None:
                  _wellOriginX = self._field_info.wellOriginX.wrapper(_wellOriginX)
          self._wellOriginX = _wellOriginX
          pass

      def unloadWellOriginY(self, ):
          self._wellOriginYLoaded = False
          self._wellOriginY = None;

      def getWellOriginY(self, current = None):
          self.errorIfUnloaded()
          return self._wellOriginY

      def setWellOriginY(self, _wellOriginY, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.wellOriginY.wrapper is not None:
              if _wellOriginY is not None:
                  _wellOriginY = self._field_info.wellOriginY.wrapper(_wellOriginY)
          self._wellOriginY = _wellOriginY
          pass

      def unloadRows(self, ):
          self._rowsLoaded = False
          self._rows = None;

      def getRows(self, current = None):
          self.errorIfUnloaded()
          return self._rows

      def setRows(self, _rows, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.rows.wrapper is not None:
              if _rows is not None:
                  _rows = self._field_info.rows.wrapper(_rows)
          self._rows = _rows
          pass

      def unloadColumns(self, ):
          self._columnsLoaded = False
          self._columns = None;

      def getColumns(self, current = None):
          self.errorIfUnloaded()
          return self._columns

      def setColumns(self, _columns, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.columns.wrapper is not None:
              if _columns is not None:
                  _columns = self._field_info.columns.wrapper(_columns)
          self._columns = _columns
          pass

      def unloadStatus(self, ):
          self._statusLoaded = False
          self._status = None;

      def getStatus(self, current = None):
          self.errorIfUnloaded()
          return self._status

      def setStatus(self, _status, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.status.wrapper is not None:
              if _status is not None:
                  _status = self._field_info.status.wrapper(_status)
          self._status = _status
          pass

      def unloadExternalIdentifier(self, ):
          self._externalIdentifierLoaded = False
          self._externalIdentifier = None;

      def getExternalIdentifier(self, current = None):
          self.errorIfUnloaded()
          return self._externalIdentifier

      def setExternalIdentifier(self, _externalIdentifier, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.externalIdentifier.wrapper is not None:
              if _externalIdentifier is not None:
                  _externalIdentifier = self._field_info.externalIdentifier.wrapper(_externalIdentifier)
          self._externalIdentifier = _externalIdentifier
          pass

      def unloadScreenLinks(self, current = None):
          self._screenLinksLoaded = False
          self._screenLinksSeq = None;

      def _getScreenLinks(self, current = None):
          self.errorIfUnloaded()
          return self._screenLinksSeq

      def _setScreenLinks(self, _screenLinks, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.screenLinksSeq.wrapper is not None:
              if _screenLinks is not None:
                  _screenLinks = self._field_info.screenLinksSeq.wrapper(_screenLinks)
          self._screenLinksSeq = _screenLinks
          self.checkUnloadedProperty(_screenLinks,'screenLinksLoaded')

      def isScreenLinksLoaded(self):
          return self._screenLinksLoaded

      def sizeOfScreenLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: return -1
          return len(self._screenLinksSeq)

      def copyScreenLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          return list(self._screenLinksSeq)

      def iterateScreenLinks(self):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          return iter(self._screenLinksSeq)

      def addScreenPlateLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          self._screenLinksSeq.append( target );
          target.setChild( self )

      def addAllScreenPlateLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          self._screenLinksSeq.extend( targets )
          for target in targets:
              target.setChild( self )

      def removeScreenPlateLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          self._screenLinksSeq.remove( target )
          target.setChild( None )

      def removeAllScreenPlateLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          for elt in targets:
              elt.setChild( None )
              self._screenLinksSeq.remove( elt )

      def clearScreenLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          for elt in self._screenLinksSeq:
              elt.setChild( None )
          self._screenLinksSeq = list()

      def reloadScreenLinks(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._screenLinksLoaded:
              raise omero.ClientError("Cannot reload active collection: screenLinksSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyScreenLinks() # May also throw
          for elt in copy:
              elt.setChild( self )
          self._screenLinksSeq = copy
          toCopy.unloadScreenLinks()
          self._screenLinksLoaded = True

      def getScreenLinksCountPerOwner(self, current = None):
          return self._screenLinksCountPerOwner

      def linkScreen(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          link = _omero_model.ScreenPlateLinkI()
          link.link( addition, self );
          self.addScreenPlateLinkToBoth( link, True )
          return link

      def addScreenPlateLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          self._screenLinksSeq.append( link )
          if bothSides and link.getParent().isLoaded():
              link.getParent().addScreenPlateLinkToBoth( link, False )

      def findScreenPlateLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          result = list()
          for link in self._screenLinksSeq:
              if link.getParent() == removal: result.append(link)
          return result

      def unlinkScreen(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          toRemove = self.findScreenPlateLink(removal)
          for next in toRemove:
              self.removeScreenPlateLinkFromBoth( next, True )

      def removeScreenPlateLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._screenLinksLoaded: self.throwNullCollectionException("screenLinksSeq")
          self._screenLinksSeq.remove( link )
          if bothSides and link.getParent().isLoaded():
              link.getParent().removeScreenPlateLinkFromBoth(link, False)

      def linkedScreenList(self, current = None):
          self.errorIfUnloaded()
          if not self.screenLinksLoaded: self.throwNullCollectionException("ScreenLinks")
          linked = []
          for link in self._screenLinksSeq:
              linked.append( link.getParent() )
          return linked

      def unloadWells(self, current = None):
          self._wellsLoaded = False
          self._wellsSeq = None;

      def _getWells(self, current = None):
          self.errorIfUnloaded()
          return self._wellsSeq

      def _setWells(self, _wells, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.wellsSeq.wrapper is not None:
              if _wells is not None:
                  _wells = self._field_info.wellsSeq.wrapper(_wells)
          self._wellsSeq = _wells
          self.checkUnloadedProperty(_wells,'wellsLoaded')

      def isWellsLoaded(self):
          return self._wellsLoaded

      def sizeOfWells(self, current = None):
          self.errorIfUnloaded()
          if not self._wellsLoaded: return -1
          return len(self._wellsSeq)

      def copyWells(self, current = None):
          self.errorIfUnloaded()
          if not self._wellsLoaded: self.throwNullCollectionException("wellsSeq")
          return list(self._wellsSeq)

      def iterateWells(self):
          self.errorIfUnloaded()
          if not self._wellsLoaded: self.throwNullCollectionException("wellsSeq")
          return iter(self._wellsSeq)

      def addWell(self, target, current = None):
          self.errorIfUnloaded()
          if not self._wellsLoaded: self.throwNullCollectionException("wellsSeq")
          self._wellsSeq.append( target );
          target.setPlate( self )

      def addAllWellSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._wellsLoaded: self.throwNullCollectionException("wellsSeq")
          self._wellsSeq.extend( targets )
          for target in targets:
              target.setPlate( self )

      def removeWell(self, target, current = None):
          self.errorIfUnloaded()
          if not self._wellsLoaded: self.throwNullCollectionException("wellsSeq")
          self._wellsSeq.remove( target )
          target.setPlate( None )

      def removeAllWellSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._wellsLoaded: self.throwNullCollectionException("wellsSeq")
          for elt in targets:
              elt.setPlate( None )
              self._wellsSeq.remove( elt )

      def clearWells(self, current = None):
          self.errorIfUnloaded()
          if not self._wellsLoaded: self.throwNullCollectionException("wellsSeq")
          for elt in self._wellsSeq:
              elt.setPlate( None )
          self._wellsSeq = list()

      def reloadWells(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._wellsLoaded:
              raise omero.ClientError("Cannot reload active collection: wellsSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyWells() # May also throw
          for elt in copy:
              elt.setPlate( self )
          self._wellsSeq = copy
          toCopy.unloadWells()
          self._wellsLoaded = True

      def unloadPlateAcquisitions(self, current = None):
          self._plateAcquisitionsLoaded = False
          self._plateAcquisitionsSeq = None;

      def _getPlateAcquisitions(self, current = None):
          self.errorIfUnloaded()
          return self._plateAcquisitionsSeq

      def _setPlateAcquisitions(self, _plateAcquisitions, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.plateAcquisitionsSeq.wrapper is not None:
              if _plateAcquisitions is not None:
                  _plateAcquisitions = self._field_info.plateAcquisitionsSeq.wrapper(_plateAcquisitions)
          self._plateAcquisitionsSeq = _plateAcquisitions
          self.checkUnloadedProperty(_plateAcquisitions,'plateAcquisitionsLoaded')

      def isPlateAcquisitionsLoaded(self):
          return self._plateAcquisitionsLoaded

      def sizeOfPlateAcquisitions(self, current = None):
          self.errorIfUnloaded()
          if not self._plateAcquisitionsLoaded: return -1
          return len(self._plateAcquisitionsSeq)

      def copyPlateAcquisitions(self, current = None):
          self.errorIfUnloaded()
          if not self._plateAcquisitionsLoaded: self.throwNullCollectionException("plateAcquisitionsSeq")
          return list(self._plateAcquisitionsSeq)

      def iteratePlateAcquisitions(self):
          self.errorIfUnloaded()
          if not self._plateAcquisitionsLoaded: self.throwNullCollectionException("plateAcquisitionsSeq")
          return iter(self._plateAcquisitionsSeq)

      def addPlateAcquisition(self, target, current = None):
          self.errorIfUnloaded()
          if not self._plateAcquisitionsLoaded: self.throwNullCollectionException("plateAcquisitionsSeq")
          self._plateAcquisitionsSeq.append( target );
          target.setPlate( self )

      def addAllPlateAcquisitionSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._plateAcquisitionsLoaded: self.throwNullCollectionException("plateAcquisitionsSeq")
          self._plateAcquisitionsSeq.extend( targets )
          for target in targets:
              target.setPlate( self )

      def removePlateAcquisition(self, target, current = None):
          self.errorIfUnloaded()
          if not self._plateAcquisitionsLoaded: self.throwNullCollectionException("plateAcquisitionsSeq")
          self._plateAcquisitionsSeq.remove( target )
          target.setPlate( None )

      def removeAllPlateAcquisitionSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._plateAcquisitionsLoaded: self.throwNullCollectionException("plateAcquisitionsSeq")
          for elt in targets:
              elt.setPlate( None )
              self._plateAcquisitionsSeq.remove( elt )

      def clearPlateAcquisitions(self, current = None):
          self.errorIfUnloaded()
          if not self._plateAcquisitionsLoaded: self.throwNullCollectionException("plateAcquisitionsSeq")
          for elt in self._plateAcquisitionsSeq:
              elt.setPlate( None )
          self._plateAcquisitionsSeq = list()

      def reloadPlateAcquisitions(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._plateAcquisitionsLoaded:
              raise omero.ClientError("Cannot reload active collection: plateAcquisitionsSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyPlateAcquisitions() # May also throw
          for elt in copy:
              elt.setPlate( self )
          self._plateAcquisitionsSeq = copy
          toCopy.unloadPlateAcquisitions()
          self._plateAcquisitionsLoaded = True

      def unloadAnnotationLinks(self, current = None):
          self._annotationLinksLoaded = False
          self._annotationLinksSeq = None;

      def _getAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          return self._annotationLinksSeq

      def _setAnnotationLinks(self, _annotationLinks, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.annotationLinksSeq.wrapper is not None:
              if _annotationLinks is not None:
                  _annotationLinks = self._field_info.annotationLinksSeq.wrapper(_annotationLinks)
          self._annotationLinksSeq = _annotationLinks
          self.checkUnloadedProperty(_annotationLinks,'annotationLinksLoaded')

      def isAnnotationLinksLoaded(self):
          return self._annotationLinksLoaded

      def sizeOfAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: return -1
          return len(self._annotationLinksSeq)

      def copyAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          return list(self._annotationLinksSeq)

      def iterateAnnotationLinks(self):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          return iter(self._annotationLinksSeq)

      def addPlateAnnotationLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.append( target );
          target.setParent( self )

      def addAllPlateAnnotationLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.extend( targets )
          for target in targets:
              target.setParent( self )

      def removePlateAnnotationLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.remove( target )
          target.setParent( None )

      def removeAllPlateAnnotationLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          for elt in targets:
              elt.setParent( None )
              self._annotationLinksSeq.remove( elt )

      def clearAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          for elt in self._annotationLinksSeq:
              elt.setParent( None )
          self._annotationLinksSeq = list()

      def reloadAnnotationLinks(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._annotationLinksLoaded:
              raise omero.ClientError("Cannot reload active collection: annotationLinksSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyAnnotationLinks() # May also throw
          for elt in copy:
              elt.setParent( self )
          self._annotationLinksSeq = copy
          toCopy.unloadAnnotationLinks()
          self._annotationLinksLoaded = True

      def getAnnotationLinksCountPerOwner(self, current = None):
          return self._annotationLinksCountPerOwner

      def linkAnnotation(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          link = _omero_model.PlateAnnotationLinkI()
          link.link( self, addition );
          self.addPlateAnnotationLinkToBoth( link, True )
          return link

      def addPlateAnnotationLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.append( link )

      def findPlateAnnotationLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          result = list()
          for link in self._annotationLinksSeq:
              if link.getChild() == removal: result.append(link)
          return result

      def unlinkAnnotation(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          toRemove = self.findPlateAnnotationLink(removal)
          for next in toRemove:
              self.removePlateAnnotationLinkFromBoth( next, True )

      def removePlateAnnotationLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.remove( link )

      def linkedAnnotationList(self, current = None):
          self.errorIfUnloaded()
          if not self.annotationLinksLoaded: self.throwNullCollectionException("AnnotationLinks")
          linked = []
          for link in self._annotationLinksSeq:
              linked.append( link.getChild() )
          return linked

      def unloadName(self, ):
          self._nameLoaded = False
          self._name = None;

      def getName(self, current = None):
          self.errorIfUnloaded()
          return self._name

      def setName(self, _name, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.name.wrapper is not None:
              if _name is not None:
                  _name = self._field_info.name.wrapper(_name)
          self._name = _name
          pass

      def unloadDescription(self, ):
          self._descriptionLoaded = False
          self._description = None;

      def getDescription(self, current = None):
          self.errorIfUnloaded()
          return self._description

      def setDescription(self, _description, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.description.wrapper is not None:
              if _description is not None:
                  _description = self._field_info.description.wrapper(_description)
          self._description = _description
          pass


      def ice_postUnmarshal(self):
          """
          Provides additional initialization once all data loaded
          """
          pass # Currently unused


      def ice_preMarshal(self):
          """
          Provides additional validation before data is sent
          """
          pass # Currently unused

      def __getattr__(self, name):
          """
          Reroutes all access to object.field through object.getField() or object.isField()
          """
          if "_" in name:  # Ice disallows underscores, so these should be treated normally.
              return object.__getattribute__(self, name)
          field  = "_" + name
          capitalized = name[0].capitalize() + name[1:]
          getter = "get" + capitalized
          questn = "is" + capitalized
          try:
              self.__dict__[field]
              if hasattr(self, getter):
                  method = getattr(self, getter)
                  return method()
              elif hasattr(self, questn):
                  method = getattr(self, questn)
                  return method()
          except:
              pass
          raise AttributeError("'%s' object has no attribute '%s' or '%s'" % (self.__class__.__name__, getter, questn))

      def __setattr__(self, name, value):
          """
          Reroutes all access to object.field through object.getField(), with the caveat
          that all sets on variables starting with "_" are permitted directly.
          """
          if name.startswith("_"):
              self.__dict__[name] = value
              return
          else:
              field  = "_" + name
              setter = "set" + name[0].capitalize() + name[1:]
              if hasattr(self, field) and hasattr(self, setter):
                  method = getattr(self, setter)
                  return method(value)
          raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, setter))

_omero_model.PlateI = PlateI



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Path.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice
import omero_model_Shape_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Roi' not in _M_omero.model.__dict__:
    _M_omero.model._t_Roi = IcePy.declareClass('::omero::model::Roi')
    _M_omero.model._t_RoiPrx = IcePy.declareProxy('::omero::model::Roi')

if 'AffineTransform' not in _M_omero.model.__dict__:
    _M_omero.model._t_AffineTransform = IcePy.declareClass('::omero::model::AffineTransform')
    _M_omero.model._t_AffineTransformPrx = IcePy.declareProxy('::omero::model::AffineTransform')

if 'Length' not in _M_omero.model.__dict__:
    _M_omero.model._t_Length = IcePy.declareClass('::omero::model::Length')
    _M_omero.model._t_LengthPrx = IcePy.declareProxy('::omero::model::Length')

if 'ShapeAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_ShapeAnnotationLink = IcePy.declareClass('::omero::model::ShapeAnnotationLink')
    _M_omero.model._t_ShapeAnnotationLinkPrx = IcePy.declareProxy('::omero::model::ShapeAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'Path' not in _M_omero.model.__dict__:
    _M_omero.model.Path = Ice.createTempClass()
    class Path(_M_omero.model.Shape):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _theZ=None, _theT=None, _theC=None, _roi=None, _locked=None, _transform=None, _fillColor=None, _fillRule=None, _strokeColor=None, _strokeDashArray=None, _strokeWidth=None, _fontFamily=None, _fontSize=None, _fontStyle=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None, _d=None, _textValue=None):
            if Ice.getType(self) == _M_omero.model.Path:
                raise RuntimeError('omero.model.Path is an abstract class')
            _M_omero.model.Shape.__init__(self, _id, _details, _loaded, _version, _theZ, _theT, _theC, _roi, _locked, _transform, _fillColor, _fillRule, _strokeColor, _strokeDashArray, _strokeWidth, _fontFamily, _fontSize, _fontStyle, _annotationLinksSeq, _annotationLinksLoaded, _annotationLinksCountPerOwner)
            self._d = _d
            self._textValue = _textValue

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Path', '::omero::model::Shape')

        def ice_id(self, current=None):
            return '::omero::model::Path'

        def ice_staticId():
            return '::omero::model::Path'
        ice_staticId = staticmethod(ice_staticId)

        def getD(self, current=None):
            pass

        def setD(self, theD, current=None):
            pass

        def getTextValue(self, current=None):
            pass

        def setTextValue(self, theTextValue, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Path)

        __repr__ = __str__

    _M_omero.model.PathPrx = Ice.createTempClass()
    class PathPrx(_M_omero.model.ShapePrx):

        def getD(self, _ctx=None):
            return _M_omero.model.Path._op_getD.invoke(self, ((), _ctx))

        def begin_getD(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Path._op_getD.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getD(self, _r):
            return _M_omero.model.Path._op_getD.end(self, _r)

        def setD(self, theD, _ctx=None):
            return _M_omero.model.Path._op_setD.invoke(self, ((theD, ), _ctx))

        def begin_setD(self, theD, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Path._op_setD.begin(self, ((theD, ), _response, _ex, _sent, _ctx))

        def end_setD(self, _r):
            return _M_omero.model.Path._op_setD.end(self, _r)

        def getTextValue(self, _ctx=None):
            return _M_omero.model.Path._op_getTextValue.invoke(self, ((), _ctx))

        def begin_getTextValue(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Path._op_getTextValue.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getTextValue(self, _r):
            return _M_omero.model.Path._op_getTextValue.end(self, _r)

        def setTextValue(self, theTextValue, _ctx=None):
            return _M_omero.model.Path._op_setTextValue.invoke(self, ((theTextValue, ), _ctx))

        def begin_setTextValue(self, theTextValue, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Path._op_setTextValue.begin(self, ((theTextValue, ), _response, _ex, _sent, _ctx))

        def end_setTextValue(self, _r):
            return _M_omero.model.Path._op_setTextValue.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.PathPrx.ice_checkedCast(proxy, '::omero::model::Path', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.PathPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Path'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_PathPrx = IcePy.defineProxy('::omero::model::Path', PathPrx)

    _M_omero.model._t_Path = IcePy.declareClass('::omero::model::Path')

    _M_omero.model._t_Path = IcePy.defineClass('::omero::model::Path', Path, -1, (), True, False, _M_omero.model._t_Shape, (), (
        ('_d', (), _M_omero._t_RString, False, 0),
        ('_textValue', (), _M_omero._t_RString, False, 0)
    ))
    Path._ice_type = _M_omero.model._t_Path

    Path._op_getD = IcePy.Operation('getD', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Path._op_getD.deprecate("Path is deprecated.")
    Path._op_setD = IcePy.Operation('setD', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Path._op_setD.deprecate("Path is deprecated.")
    Path._op_getTextValue = IcePy.Operation('getTextValue', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Path._op_getTextValue.deprecate("Path is deprecated.")
    Path._op_setTextValue = IcePy.Operation('setTextValue', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Path._op_setTextValue.deprecate("Path is deprecated.")

    _M_omero.model.Path = Path
    del Path

    _M_omero.model.PathPrx = PathPrx
    del PathPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import object
import sys
sys.path.append('.')

import omero.gateway
import omero.model
import os
import subprocess
import urllib.request, urllib.error, urllib.parse

try:
    from types import StringTypes
except ImportError:
    StringTypes = str

from omero_ext.path import path

BASEPATH = os.path.dirname(os.path.abspath(__file__))
TESTIMG_URL = 'https://downloads.openmicroscopy.org/images/gateway_tests/'
DEFAULT_GROUP_PERMS = 'rwr---'

if not omero.gateway.BlitzGateway.ICE_CONFIG:
    try:
        import settings
        iceconfig = os.environ.get('ICE_CONFIG', None)
        if iceconfig is None:
            iceconfig = os.path.join(settings.OMERO_HOME, 'etc', 'ice.config')
        omero.gateway.BlitzGateway.ICE_CONFIG = iceconfig
    except ImportError:
        pass
    except AttributeError:
        pass

# Gateway = omero.gateway.BlitzGateway


def refreshConfig():
    bg = omero.gateway.BlitzGateway()
    try:
        ru = bg.c.ic.getProperties().getProperty('omero.rootuser')
        rp = bg.c.ic.getProperties().getProperty('omero.rootpass')
    finally:
        bg.close()

    if ru:
        ROOT.name = ru
    if rp:
        ROOT.passwd = rp


def loginAsRoot():
    refreshConfig()
    return login(ROOT)


def loginAsPublic():
    return login(settings.PUBLIC_USER, settings.PUBLIC_PASSWORD)


def login(alias, pw=None, groupname=None):
    if isinstance(alias, UserEntry):
        return alias.login(groupname=groupname)
    elif pw is None:
        return USERS[alias].login(groupname=groupname)
    else:
        return UserEntry(alias, pw).login(groupname=groupname)


class BadGroupPermissionsException(Exception):
    pass


class UserEntry (object):

    def __init__(self, name, passwd, firstname='', middlename='', lastname='',
                 email='', ldap=False, groupname=None, groupperms=None,
                 groupowner=False, admin=False):
        """
        If no groupperms are passed, then check_group_perms will do nothing.
        The default perms for newly created groups is defined
        in _getOrCreateGroup
        """
        self.name = name
        self.passwd = passwd
        self.firstname = firstname
        self.middlename = middlename
        self.lastname = lastname
        self.ldap = ldap
        self.email = email
        self.admin = admin
        self.groupname = groupname
        self.groupperms = groupperms
        self.groupowner = groupowner

    def fullname(self):
        return '%s %s' % (self.firstname, self.lastname)

    def login(self, groupname=None):
        if groupname is None:
            groupname = self.groupname
        client = omero.gateway.BlitzGateway(
            self.name, self.passwd, group=groupname, try_super=self.admin)
        if not client.connect():
            print("Can not connect")
            return None

        a = client.getAdminService()
        if groupname is not None:
            if client.getEventContext().groupName != groupname:
                try:
                    g = a.lookupGroup(groupname)
                    client.setGroupForSession(g.getId().val)
                except:
                    pass

        # Reset group name and evaluate
        self.groupname = a.getEventContext().groupName
        if self.groupname != "system":
            UserEntry.check_group_perms(
                client, self.groupname, self.groupperms)

        return client

    @staticmethod
    def check_group_perms(client, group, groupperms):
        """
        If expected permissions have been set, then this will
        enforce equality. If groupperms are None, then
        nothing will be checked.
        """
        if groupperms is not None:
            if isinstance(group, StringTypes):
                a = client.getAdminService()
                g = a.lookupGroup(group)
            else:
                g = group
            p = g.getDetails().getPermissions()
            if str(p) != groupperms:
                raise BadGroupPermissionsException(
                    "%s group has wrong permissions! Expected: %s Found: %s" %
                    (g.getName(), groupperms, p))

    @staticmethod
    def assert_group_perms(client, group, groupperms):
        """
        If expected permissions have been set, then this will
        change group permissions to those requested if not
        already equal. If groupperms are None, then
        nothing will be checked.
        """
        a = client.getAdminService()
        try:
            if isinstance(group, StringTypes):
                g = a.lookupGroup(group)
            else:
                g = group
            UserEntry.check_group_perms(client, g, groupperms)
        except BadGroupPermissionsException:
            client._waitOnCmd(client.chmodGroup(g.id.val, groupperms))

    @staticmethod
    def _getOrCreateGroup(client, groupname, ldap=False, groupperms=None):

        # Default on class is None
        if groupperms is None:
            groupperms = DEFAULT_GROUP_PERMS

        a = client.getAdminService()
        try:
            g = a.lookupGroup(groupname)
        except:
            g = omero.model.ExperimenterGroupI()
            g.setName(omero.gateway.omero_type(groupname))
            g.setLdap(omero.gateway.omero_type(ldap))
            p = omero.model.PermissionsI(groupperms)
            g.details.setPermissions(p)
            a.createGroup(g)
            g = a.lookupGroup(groupname)
        UserEntry.check_group_perms(client, groupname, groupperms)
        return g

    def create(self, client, password):
        a = client.getAdminService()
        try:
            a.lookupExperimenter(self.name)
            # print "Already exists: %s" % self.name
            return False
        except:
            # print "Creating: %s" % self.name
            pass
        if self.groupname is None:
            self.groupname = self.name + '_group'
        g = UserEntry._getOrCreateGroup(
            client, self.groupname, groupperms=self.groupperms)
        u = omero.model.ExperimenterI()
        u.setOmeName(omero.gateway.omero_type(self.name))
        u.setFirstName(omero.gateway.omero_type(self.firstname))
        u.setMiddleName(omero.gateway.omero_type(self.middlename))
        u.setLastName(omero.gateway.omero_type(self.lastname))
        u.setLdap(omero.gateway.omero_type(self.ldap))
        u.setEmail(omero.gateway.omero_type(self.email))
        a.createUser(u, g.getName().val)
        u = a.lookupExperimenter(self.name)
        if self.admin:
            a.addGroups(u, (a.lookupGroup("system"),))
        client.c.sf.setSecurityPassword(password)  # See #3202
        a.changeUserPassword(
            u.getOmeName().val, omero.gateway.omero_type(self.passwd))
        if self.groupowner:
            a.setGroupOwner(g, u)
        return True

    def changePassword(self, client, password, rootpass):
        a = client.getAdminService()
        client.c.sf.setSecurityPassword(rootpass)  # See #3202
        a.changeUserPassword(self.name, omero.gateway.omero_type(password))

    @staticmethod
    def addGroupToUser(client, groupname, groupperms=None):
        if groupperms is None:
            groupperms = DEFAULT_GROUP_PERMS

        a = client.getAdminService()
        admin_gateway = None
        try:
            if 'system' not in [x.name.val for x in a.containedGroups(
                    client.getUserId())]:
                admin_gateway = loginAsRoot()
                a = admin_gateway.getAdminService()
            g = UserEntry._getOrCreateGroup(
                client, groupname, groupperms=groupperms)
            a.addGroups(a.getExperimenter(client.getUserId()), (g,))
        finally:
            # Always clean up the results of login
            if admin_gateway:
                admin_gateway.close()

    @staticmethod
    def setGroupForSession(client, groupname, groupperms=None):
        if groupperms is None:
            groupperms = DEFAULT_GROUP_PERMS

        a = client.getAdminService()
        if groupname not in [x.name.val for x in a.containedGroups(
                client.getUserId())]:
            UserEntry.addGroupToUser(client, groupname, groupperms)
            # Must reconnect to read new groupexperimentermap
            t = client.clone()
            client.c.closeSession()
            client._proxies = omero.gateway.NoProxies()
            client._ctx = None
            client.c = t.c
            client.connect()
            a = client.getAdminService()
        g = a.lookupGroup(groupname)
        client.setGroupForSession(g.getId().val)
        return client


class ObjectEntry (object):
    pass


class ProjectEntry (ObjectEntry):

    def __init__(self, name, owner, create_group=False, group_perms=None):
        self.name = name
        self.owner = owner
        self.create_group = create_group
        self.group_perms = group_perms

    def get(self, client=None, fromCreate=False):
        if client is None:
            client = USERS[self.owner].login()
        for p in client.listProjects():
            if p.getName() == self.name:
                p.__loadedHotSwap__()
                return p
        return None

    def create(self, client=None):
        if client is None:
            client = USERS[self.owner].login()
        p = self.get(client)
        if p is not None:
            return p
        p = omero.model.ProjectI(loaded=True)
        p.setName(omero.gateway.omero_type(self.name))
        p.setDescription(omero.gateway.omero_type(self.name))
        if self.create_group:
            if isinstance(self.create_group, StringTypes):
                groupname = self.create_group
            else:
                raise ValueError('group must be string')
                groupname = 'project_test'

            s = loginAsRoot()
            UserEntry._getOrCreateGroup(
                s, groupname, groupperms=self.group_perms)
            try:
                UserEntry.addGroupToUser(s, groupname, self.group_perms)
            finally:
                s.close()

            UserEntry.setGroupForSession(client, groupname, self.group_perms)
        p = omero.gateway.ProjectWrapper(
            client, client.getUpdateService().saveAndReturnObject(p))
        return self.get(client, True)


class DatasetEntry (ObjectEntry):

    def __init__(self, name, project, description=None, callback=None):
        self.name = name
        self.project = project
        self.description = description
        self.callback = callback

    def get(self, client, forceproj=None):
        if forceproj is None:
            if isinstance(self.project, StringTypes):
                project = PROJECTS[self.project].get(client)
            elif isinstance(self.project, ProjectEntry):
                project = self.project.get(client)
            else:
                project = self.project
        else:
            project = forceproj
        for d in project.listChildren():
            if d.getName() == self.name and self.description_check(d):
                d.__loadedHotSwap__()
                return d
        return None

    def create(self):
        if isinstance(self.project, StringTypes):
            project = PROJECTS[self.project]
            user = USERS[project.owner]
            client = user.login()
            project = project.get(client)
        else:
            project = self.project
            client = project._conn
        d = self.get(client, project)
        if d is not None and self.description_check(d):
            return d
        d = omero.model.DatasetI(loaded=True)
        d.setName(omero.gateway.omero_type(self.name))
        if self.description is not None:
            d.setDescription(omero.gateway.omero_type(self.description))
        project.linkDataset(d)
        project.save()
        rv = self.get(client, project)
        if self.callback:
            self.callback(rv)
        return rv

    def description_check(self, d):
        desc_match = (
            omero.gateway.omero_type(d.getDescription()) ==
            omero.gateway.omero_type(self.description))
        desc_check = (
            (self.description is None and d.getDescription() == '')
            or (self.description is not None and desc_match))
        return desc_check


class ImageEntry (ObjectEntry):

    def __init__(self, name, filename, dataset, callback=None):
        self.name = name
        self.filename = filename  # If False will create image without pixels
        if self.name is None and filename:
            self.name = os.path.basename(filename)
        self.dataset = dataset
        self.callback = callback

    def get(self, client, forceds=None):
        if forceds is None:
            dataset = DATASETS[self.dataset].get(client)
        else:
            dataset = forceds
        for i in dataset.listChildren():
            if i.getName() == self.name:
                return i
        return None

    def create(self):
        if isinstance(self.dataset, StringTypes):
            dataset = DATASETS[self.dataset]
            project = PROJECTS[dataset.project]
            client = USERS[project.owner].login()
            dataset = dataset.get(client)
        else:
            dataset = self.dataset
            client = dataset._conn
        i = self.get(client, dataset)
        if i is not None:
            # print ".. -> image already exists: %s" % self.name
            return i
        # print ".. -> create new image: %s" % self.name
        sys.stderr.write('I')
        if self.filename is False:
            UserEntry.setGroupForSession(
                client, dataset.getDetails().getGroup().getName())
            self._createWithoutPixels(client, dataset)
            return self.get(client, dataset)
        fpath = os.path.join(BASEPATH, self.filename)
        if not os.path.exists(fpath):
            if not os.path.exists(os.path.dirname(fpath)):
                os.makedirs(os.path.dirname(fpath))
            if self.filename.endswith('.fake'):
                # If it's a .fake file, simply create it
                os.close(os.open(fpath, os.O_CREAT | os.O_EXCL))
            else:
                # First try to download the image
                try:
                    # print "Trying to get test image from " + TESTIMG_URL +
                    # self.filename
                    sys.stderr.write('<')
                    fin = urllib.request.urlopen(TESTIMG_URL + self.filename)
                    with open(fpath, 'wb') as fout:
                        fout.write(fin.read())
                except urllib.error.HTTPError:
                    raise IOError('No such file %s' % fpath)
        host = dataset._conn.c.ic.getProperties().getProperty(
            'omero.host') or 'localhost'
        port = dataset._conn.c.ic.getProperties().getProperty(
            'omero.port') or '4063'

        exe = "omero"
        newconn = dataset._conn.clone()
        newconn.connect()
        try:
            UserEntry.setGroupForSession(
                newconn, dataset.getDetails().getGroup().getName())
            session = newconn._sessionUuid
            # print session
            exe += ' -s %s -k %s -p %s import -d %i --output legacy -n' % (
                host, session, port, dataset.getId())
            exe = exe.split() + [self.name, fpath]
            print(' '.join(exe))
            try:
                p = subprocess.Popen(
                    exe,  shell=False, stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE)
            except OSError:
                print("!!Please make sure the 'omero' executable is in PATH")
                return None

            pid = p.communicate()
            try:
                img = omero.gateway.ImageWrapper(
                    dataset._conn,
                    dataset._conn.getQueryService().find(
                        'Pixels', int(pid[0].split(b'\n')[0].strip())).image)
            except ValueError:
                print(pid)
                raise
            # print "imgid = %i" % img.getId()
            img.setName(self.name)
            # img._obj.objectiveSettings = None
            img.save()
            if self.callback:
                self.callback(img)
            return img
        finally:
            # Always cleanup the return from clone/connect
            newconn.close()

    def _createWithoutPixels(self, client, dataset):
        img = omero.model.ImageI()
        img.setName(omero.gateway.omero_type(self.name))
        if not dataset.imageLinksLoaded:
            print(".!.")
            dataset._obj._imageLinksSeq = []
            dataset._obj._imageLinksLoaded = True
        dataset.linkImage(img)
        dataset.save()


def getProject(client, alias):
    return PROJECTS[alias].get(client)


def assertCommentAnnotation(object, ns, value):
    ann = object.getAnnotation(ns)
    if ann is None or ann.getValue() != value:
        ann = omero.gateway.CommentAnnotationWrapper()
        ann.setNs(ns)
        ann.setValue(value)
        object.linkAnnotation(ann)
    return ann


def getDataset(client, alias, forceproj=None):
    return DATASETS[alias].get(client, forceproj)


def getImage(client, alias, forceds=None, autocreate=False):
    rv = IMAGES[alias].get(client, forceds)
    if rv is None and autocreate:
        i = IMAGES[alias].create()
        i._conn.close()
        rv = IMAGES[alias].get(client, forceds)
    return rv


def bootstrap(onlyUsers=False, skipImages=True):
    # Create users
    client = loginAsRoot()
    try:
        for k, u in list(USERS.items()):
            if not u.create(client, ROOT.passwd):
                u.changePassword(client, u.passwd, ROOT.passwd)
                u.assert_group_perms(client, u.groupname, u.groupperms)
        if onlyUsers:
            return
        for k, p in list(PROJECTS.items()):
            p = p.create()
            p._conn.close()
            # print p.get(client).getDetails().getPermissions().isUserWrite()
        for k, d in list(DATASETS.items()):
            d = d.create()
            d._conn.close()
        if not skipImages:
            for k, i in list(IMAGES.items()):
                i = i.create()
                i._conn.close()
    finally:
        client.close()


def cleanup():
    for k, p in list(PROJECTS.items()):
        sys.stderr.write('*')
        p = p.get()
        if p is not None:
            client = p._conn
            handle = client.deleteObjects(
                'Project', [p.getId()], deleteAnns=True, deleteChildren=True)
            try:
                client._waitOnCmd(handle)
            finally:
                handle.close()
    client.close()
    client = loginAsRoot()
    for k, u in list(USERS.items()):
        u.changePassword(client, None, ROOT.passwd)
    client.close()

ROOT = UserEntry('root', 'ome', admin=True)

USERS = {
    # 'alias': UserEntry entry,
}

PROJECTS = {
    # 'alias': ProjectEntry entry,
}

DATASETS = {
    # 'alias': DatasetEntry entry,
}

IMAGES = {
    # 'alias': ImageEntry entry,
}



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `AffineTransform.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'AffineTransform' not in _M_omero.model.__dict__:
    _M_omero.model.AffineTransform = Ice.createTempClass()
    class AffineTransform(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _a00=None, _a10=None, _a01=None, _a11=None, _a02=None, _a12=None):
            if Ice.getType(self) == _M_omero.model.AffineTransform:
                raise RuntimeError('omero.model.AffineTransform is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._a00 = _a00
            self._a10 = _a10
            self._a01 = _a01
            self._a11 = _a11
            self._a02 = _a02
            self._a12 = _a12

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::AffineTransform', '::omero::model::IObject')

        def ice_id(self, current=None):
            return '::omero::model::AffineTransform'

        def ice_staticId():
            return '::omero::model::AffineTransform'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getA00(self, current=None):
            pass

        def setA00(self, theA00, current=None):
            pass

        def getA10(self, current=None):
            pass

        def setA10(self, theA10, current=None):
            pass

        def getA01(self, current=None):
            pass

        def setA01(self, theA01, current=None):
            pass

        def getA11(self, current=None):
            pass

        def setA11(self, theA11, current=None):
            pass

        def getA02(self, current=None):
            pass

        def setA02(self, theA02, current=None):
            pass

        def getA12(self, current=None):
            pass

        def setA12(self, theA12, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_AffineTransform)

        __repr__ = __str__

    _M_omero.model.AffineTransformPrx = Ice.createTempClass()
    class AffineTransformPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.AffineTransform._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.AffineTransform._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.AffineTransform._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.AffineTransform._op_setVersion.end(self, _r)

        def getA00(self, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA00.invoke(self, ((), _ctx))

        def begin_getA00(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA00.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getA00(self, _r):
            return _M_omero.model.AffineTransform._op_getA00.end(self, _r)

        def setA00(self, theA00, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA00.invoke(self, ((theA00, ), _ctx))

        def begin_setA00(self, theA00, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA00.begin(self, ((theA00, ), _response, _ex, _sent, _ctx))

        def end_setA00(self, _r):
            return _M_omero.model.AffineTransform._op_setA00.end(self, _r)

        def getA10(self, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA10.invoke(self, ((), _ctx))

        def begin_getA10(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA10.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getA10(self, _r):
            return _M_omero.model.AffineTransform._op_getA10.end(self, _r)

        def setA10(self, theA10, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA10.invoke(self, ((theA10, ), _ctx))

        def begin_setA10(self, theA10, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA10.begin(self, ((theA10, ), _response, _ex, _sent, _ctx))

        def end_setA10(self, _r):
            return _M_omero.model.AffineTransform._op_setA10.end(self, _r)

        def getA01(self, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA01.invoke(self, ((), _ctx))

        def begin_getA01(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA01.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getA01(self, _r):
            return _M_omero.model.AffineTransform._op_getA01.end(self, _r)

        def setA01(self, theA01, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA01.invoke(self, ((theA01, ), _ctx))

        def begin_setA01(self, theA01, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA01.begin(self, ((theA01, ), _response, _ex, _sent, _ctx))

        def end_setA01(self, _r):
            return _M_omero.model.AffineTransform._op_setA01.end(self, _r)

        def getA11(self, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA11.invoke(self, ((), _ctx))

        def begin_getA11(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA11.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getA11(self, _r):
            return _M_omero.model.AffineTransform._op_getA11.end(self, _r)

        def setA11(self, theA11, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA11.invoke(self, ((theA11, ), _ctx))

        def begin_setA11(self, theA11, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA11.begin(self, ((theA11, ), _response, _ex, _sent, _ctx))

        def end_setA11(self, _r):
            return _M_omero.model.AffineTransform._op_setA11.end(self, _r)

        def getA02(self, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA02.invoke(self, ((), _ctx))

        def begin_getA02(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA02.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getA02(self, _r):
            return _M_omero.model.AffineTransform._op_getA02.end(self, _r)

        def setA02(self, theA02, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA02.invoke(self, ((theA02, ), _ctx))

        def begin_setA02(self, theA02, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA02.begin(self, ((theA02, ), _response, _ex, _sent, _ctx))

        def end_setA02(self, _r):
            return _M_omero.model.AffineTransform._op_setA02.end(self, _r)

        def getA12(self, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA12.invoke(self, ((), _ctx))

        def begin_getA12(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_getA12.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getA12(self, _r):
            return _M_omero.model.AffineTransform._op_getA12.end(self, _r)

        def setA12(self, theA12, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA12.invoke(self, ((theA12, ), _ctx))

        def begin_setA12(self, theA12, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.AffineTransform._op_setA12.begin(self, ((theA12, ), _response, _ex, _sent, _ctx))

        def end_setA12(self, _r):
            return _M_omero.model.AffineTransform._op_setA12.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.AffineTransformPrx.ice_checkedCast(proxy, '::omero::model::AffineTransform', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.AffineTransformPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::AffineTransform'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_AffineTransformPrx = IcePy.defineProxy('::omero::model::AffineTransform', AffineTransformPrx)

    _M_omero.model._t_AffineTransform = IcePy.declareClass('::omero::model::AffineTransform')

    _M_omero.model._t_AffineTransform = IcePy.defineClass('::omero::model::AffineTransform', AffineTransform, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_a00', (), _M_omero._t_RDouble, False, 0),
        ('_a10', (), _M_omero._t_RDouble, False, 0),
        ('_a01', (), _M_omero._t_RDouble, False, 0),
        ('_a11', (), _M_omero._t_RDouble, False, 0),
        ('_a02', (), _M_omero._t_RDouble, False, 0),
        ('_a12', (), _M_omero._t_RDouble, False, 0)
    ))
    AffineTransform._ice_type = _M_omero.model._t_AffineTransform

    AffineTransform._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    AffineTransform._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    AffineTransform._op_getA00 = IcePy.Operation('getA00', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    AffineTransform._op_setA00 = IcePy.Operation('setA00', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    AffineTransform._op_getA10 = IcePy.Operation('getA10', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    AffineTransform._op_setA10 = IcePy.Operation('setA10', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    AffineTransform._op_getA01 = IcePy.Operation('getA01', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    AffineTransform._op_setA01 = IcePy.Operation('setA01', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    AffineTransform._op_getA11 = IcePy.Operation('getA11', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    AffineTransform._op_setA11 = IcePy.Operation('setA11', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    AffineTransform._op_getA02 = IcePy.Operation('getA02', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    AffineTransform._op_setA02 = IcePy.Operation('setA02', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    AffineTransform._op_getA12 = IcePy.Operation('getA12', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    AffineTransform._op_setA12 = IcePy.Operation('setA12', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())

    _M_omero.model.AffineTransform = AffineTransform
    del AffineTransform

    _M_omero.model.AffineTransformPrx = AffineTransformPrx
    del AffineTransformPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Fileset.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'FilesetEntry' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetEntry = IcePy.declareClass('::omero::model::FilesetEntry')
    _M_omero.model._t_FilesetEntryPrx = IcePy.declareProxy('::omero::model::FilesetEntry')

if 'Image' not in _M_omero.model.__dict__:
    _M_omero.model._t_Image = IcePy.declareClass('::omero::model::Image')
    _M_omero.model._t_ImagePrx = IcePy.declareProxy('::omero::model::Image')

if 'FilesetJobLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetJobLink = IcePy.declareClass('::omero::model::FilesetJobLink')
    _M_omero.model._t_FilesetJobLinkPrx = IcePy.declareProxy('::omero::model::FilesetJobLink')

if 'Job' not in _M_omero.model.__dict__:
    _M_omero.model._t_Job = IcePy.declareClass('::omero::model::Job')
    _M_omero.model._t_JobPrx = IcePy.declareProxy('::omero::model::Job')

if 'FilesetAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetAnnotationLink = IcePy.declareClass('::omero::model::FilesetAnnotationLink')
    _M_omero.model._t_FilesetAnnotationLinkPrx = IcePy.declareProxy('::omero::model::FilesetAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_FilesetUsedFilesSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetUsedFilesSeq = IcePy.defineSequence('::omero::model::FilesetUsedFilesSeq', (), _M_omero.model._t_FilesetEntry)

if '_t_FilesetImagesSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetImagesSeq = IcePy.defineSequence('::omero::model::FilesetImagesSeq', (), _M_omero.model._t_Image)

if '_t_FilesetJobLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetJobLinksSeq = IcePy.defineSequence('::omero::model::FilesetJobLinksSeq', (), _M_omero.model._t_FilesetJobLink)

if '_t_FilesetLinkedJobSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetLinkedJobSeq = IcePy.defineSequence('::omero::model::FilesetLinkedJobSeq', (), _M_omero.model._t_Job)

if '_t_FilesetAnnotationLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetAnnotationLinksSeq = IcePy.defineSequence('::omero::model::FilesetAnnotationLinksSeq', (), _M_omero.model._t_FilesetAnnotationLink)

if '_t_FilesetLinkedAnnotationSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilesetLinkedAnnotationSeq = IcePy.defineSequence('::omero::model::FilesetLinkedAnnotationSeq', (), _M_omero.model._t_Annotation)

if 'Fileset' not in _M_omero.model.__dict__:
    _M_omero.model.Fileset = Ice.createTempClass()
    class Fileset(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _usedFilesSeq=None, _usedFilesLoaded=False, _imagesSeq=None, _imagesLoaded=False, _jobLinksSeq=None, _jobLinksLoaded=False, _jobLinksCountPerOwner=None, _templatePrefix=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None):
            if Ice.getType(self) == _M_omero.model.Fileset:
                raise RuntimeError('omero.model.Fileset is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._usedFilesSeq = _usedFilesSeq
            self._usedFilesLoaded = _usedFilesLoaded
            self._imagesSeq = _imagesSeq
            self._imagesLoaded = _imagesLoaded
            self._jobLinksSeq = _jobLinksSeq
            self._jobLinksLoaded = _jobLinksLoaded
            self._jobLinksCountPerOwner = _jobLinksCountPerOwner
            self._templatePrefix = _templatePrefix
            self._annotationLinksSeq = _annotationLinksSeq
            self._annotationLinksLoaded = _annotationLinksLoaded
            self._annotationLinksCountPerOwner = _annotationLinksCountPerOwner

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::Fileset', '::omero::model::IObject')

        def ice_id(self, current=None):
            return '::omero::model::Fileset'

        def ice_staticId():
            return '::omero::model::Fileset'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def unloadUsedFiles(self, current=None):
            pass

        def sizeOfUsedFiles(self, current=None):
            pass

        def copyUsedFiles(self, current=None):
            pass

        def addFilesetEntry(self, target, current=None):
            pass

        def addAllFilesetEntrySet(self, targets, current=None):
            pass

        def removeFilesetEntry(self, theTarget, current=None):
            pass

        def removeAllFilesetEntrySet(self, targets, current=None):
            pass

        def clearUsedFiles(self, current=None):
            pass

        def reloadUsedFiles(self, toCopy, current=None):
            pass

        def getFilesetEntry(self, index, current=None):
            pass

        def setFilesetEntry(self, index, theElement, current=None):
            pass

        def getPrimaryFilesetEntry(self, current=None):
            pass

        def setPrimaryFilesetEntry(self, theElement, current=None):
            pass

        def unloadImages(self, current=None):
            pass

        def sizeOfImages(self, current=None):
            pass

        def copyImages(self, current=None):
            pass

        def addImage(self, target, current=None):
            pass

        def addAllImageSet(self, targets, current=None):
            pass

        def removeImage(self, theTarget, current=None):
            pass

        def removeAllImageSet(self, targets, current=None):
            pass

        def clearImages(self, current=None):
            pass

        def reloadImages(self, toCopy, current=None):
            pass

        def unloadJobLinks(self, current=None):
            pass

        def sizeOfJobLinks(self, current=None):
            pass

        def copyJobLinks(self, current=None):
            pass

        def addFilesetJobLink(self, target, current=None):
            pass

        def addAllFilesetJobLinkSet(self, targets, current=None):
            pass

        def removeFilesetJobLink(self, theTarget, current=None):
            pass

        def removeAllFilesetJobLinkSet(self, targets, current=None):
            pass

        def clearJobLinks(self, current=None):
            pass

        def reloadJobLinks(self, toCopy, current=None):
            pass

        def getFilesetJobLink(self, index, current=None):
            pass

        def setFilesetJobLink(self, index, theElement, current=None):
            pass

        def getPrimaryFilesetJobLink(self, current=None):
            pass

        def setPrimaryFilesetJobLink(self, theElement, current=None):
            pass

        def getJobLinksCountPerOwner(self, current=None):
            pass

        def linkJob(self, addition, current=None):
            pass

        def addFilesetJobLinkToBoth(self, link, bothSides, current=None):
            pass

        def findFilesetJobLink(self, removal, current=None):
            pass

        def unlinkJob(self, removal, current=None):
            pass

        def removeFilesetJobLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedJobList(self, current=None):
            pass

        def getTemplatePrefix(self, current=None):
            pass

        def setTemplatePrefix(self, theTemplatePrefix, current=None):
            pass

        def unloadAnnotationLinks(self, current=None):
            pass

        def sizeOfAnnotationLinks(self, current=None):
            pass

        def copyAnnotationLinks(self, current=None):
            pass

        def addFilesetAnnotationLink(self, target, current=None):
            pass

        def addAllFilesetAnnotationLinkSet(self, targets, current=None):
            pass

        def removeFilesetAnnotationLink(self, theTarget, current=None):
            pass

        def removeAllFilesetAnnotationLinkSet(self, targets, current=None):
            pass

        def clearAnnotationLinks(self, current=None):
            pass

        def reloadAnnotationLinks(self, toCopy, current=None):
            pass

        def getAnnotationLinksCountPerOwner(self, current=None):
            pass

        def linkAnnotation(self, addition, current=None):
            pass

        def addFilesetAnnotationLinkToBoth(self, link, bothSides, current=None):
            pass

        def findFilesetAnnotationLink(self, removal, current=None):
            pass

        def unlinkAnnotation(self, removal, current=None):
            pass

        def removeFilesetAnnotationLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedAnnotationList(self, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Fileset)

        __repr__ = __str__

    _M_omero.model.FilesetPrx = Ice.createTempClass()
    class FilesetPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Fileset._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Fileset._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Fileset._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Fileset._op_setVersion.end(self, _r)

        def unloadUsedFiles(self, _ctx=None):
            return _M_omero.model.Fileset._op_unloadUsedFiles.invoke(self, ((), _ctx))

        def begin_unloadUsedFiles(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_unloadUsedFiles.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadUsedFiles(self, _r):
            return _M_omero.model.Fileset._op_unloadUsedFiles.end(self, _r)

        def sizeOfUsedFiles(self, _ctx=None):
            return _M_omero.model.Fileset._op_sizeOfUsedFiles.invoke(self, ((), _ctx))

        def begin_sizeOfUsedFiles(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_sizeOfUsedFiles.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfUsedFiles(self, _r):
            return _M_omero.model.Fileset._op_sizeOfUsedFiles.end(self, _r)

        def copyUsedFiles(self, _ctx=None):
            return _M_omero.model.Fileset._op_copyUsedFiles.invoke(self, ((), _ctx))

        def begin_copyUsedFiles(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_copyUsedFiles.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyUsedFiles(self, _r):
            return _M_omero.model.Fileset._op_copyUsedFiles.end(self, _r)

        def addFilesetEntry(self, target, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetEntry.invoke(self, ((target, ), _ctx))

        def begin_addFilesetEntry(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetEntry.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFilesetEntry(self, _r):
            return _M_omero.model.Fileset._op_addFilesetEntry.end(self, _r)

        def addAllFilesetEntrySet(self, targets, _ctx=None):
            return _M_omero.model.Fileset._op_addAllFilesetEntrySet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFilesetEntrySet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addAllFilesetEntrySet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFilesetEntrySet(self, _r):
            return _M_omero.model.Fileset._op_addAllFilesetEntrySet.end(self, _r)

        def removeFilesetEntry(self, theTarget, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetEntry.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFilesetEntry(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetEntry.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFilesetEntry(self, _r):
            return _M_omero.model.Fileset._op_removeFilesetEntry.end(self, _r)

        def removeAllFilesetEntrySet(self, targets, _ctx=None):
            return _M_omero.model.Fileset._op_removeAllFilesetEntrySet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFilesetEntrySet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeAllFilesetEntrySet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFilesetEntrySet(self, _r):
            return _M_omero.model.Fileset._op_removeAllFilesetEntrySet.end(self, _r)

        def clearUsedFiles(self, _ctx=None):
            return _M_omero.model.Fileset._op_clearUsedFiles.invoke(self, ((), _ctx))

        def begin_clearUsedFiles(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_clearUsedFiles.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearUsedFiles(self, _r):
            return _M_omero.model.Fileset._op_clearUsedFiles.end(self, _r)

        def reloadUsedFiles(self, toCopy, _ctx=None):
            return _M_omero.model.Fileset._op_reloadUsedFiles.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadUsedFiles(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_reloadUsedFiles.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadUsedFiles(self, _r):
            return _M_omero.model.Fileset._op_reloadUsedFiles.end(self, _r)

        def getFilesetEntry(self, index, _ctx=None):
            return _M_omero.model.Fileset._op_getFilesetEntry.invoke(self, ((index, ), _ctx))

        def begin_getFilesetEntry(self, index, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_getFilesetEntry.begin(self, ((index, ), _response, _ex, _sent, _ctx))

        def end_getFilesetEntry(self, _r):
            return _M_omero.model.Fileset._op_getFilesetEntry.end(self, _r)

        def setFilesetEntry(self, index, theElement, _ctx=None):
            return _M_omero.model.Fileset._op_setFilesetEntry.invoke(self, ((index, theElement), _ctx))

        def begin_setFilesetEntry(self, index, theElement, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_setFilesetEntry.begin(self, ((index, theElement), _response, _ex, _sent, _ctx))

        def end_setFilesetEntry(self, _r):
            return _M_omero.model.Fileset._op_setFilesetEntry.end(self, _r)

        def getPrimaryFilesetEntry(self, _ctx=None):
            return _M_omero.model.Fileset._op_getPrimaryFilesetEntry.invoke(self, ((), _ctx))

        def begin_getPrimaryFilesetEntry(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_getPrimaryFilesetEntry.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPrimaryFilesetEntry(self, _r):
            return _M_omero.model.Fileset._op_getPrimaryFilesetEntry.end(self, _r)

        def setPrimaryFilesetEntry(self, theElement, _ctx=None):
            return _M_omero.model.Fileset._op_setPrimaryFilesetEntry.invoke(self, ((theElement, ), _ctx))

        def begin_setPrimaryFilesetEntry(self, theElement, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_setPrimaryFilesetEntry.begin(self, ((theElement, ), _response, _ex, _sent, _ctx))

        def end_setPrimaryFilesetEntry(self, _r):
            return _M_omero.model.Fileset._op_setPrimaryFilesetEntry.end(self, _r)

        def unloadImages(self, _ctx=None):
            return _M_omero.model.Fileset._op_unloadImages.invoke(self, ((), _ctx))

        def begin_unloadImages(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_unloadImages.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadImages(self, _r):
            return _M_omero.model.Fileset._op_unloadImages.end(self, _r)

        def sizeOfImages(self, _ctx=None):
            return _M_omero.model.Fileset._op_sizeOfImages.invoke(self, ((), _ctx))

        def begin_sizeOfImages(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_sizeOfImages.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfImages(self, _r):
            return _M_omero.model.Fileset._op_sizeOfImages.end(self, _r)

        def copyImages(self, _ctx=None):
            return _M_omero.model.Fileset._op_copyImages.invoke(self, ((), _ctx))

        def begin_copyImages(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_copyImages.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyImages(self, _r):
            return _M_omero.model.Fileset._op_copyImages.end(self, _r)

        def addImage(self, target, _ctx=None):
            return _M_omero.model.Fileset._op_addImage.invoke(self, ((target, ), _ctx))

        def begin_addImage(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addImage.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addImage(self, _r):
            return _M_omero.model.Fileset._op_addImage.end(self, _r)

        def addAllImageSet(self, targets, _ctx=None):
            return _M_omero.model.Fileset._op_addAllImageSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllImageSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addAllImageSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllImageSet(self, _r):
            return _M_omero.model.Fileset._op_addAllImageSet.end(self, _r)

        def removeImage(self, theTarget, _ctx=None):
            return _M_omero.model.Fileset._op_removeImage.invoke(self, ((theTarget, ), _ctx))

        def begin_removeImage(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeImage.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeImage(self, _r):
            return _M_omero.model.Fileset._op_removeImage.end(self, _r)

        def removeAllImageSet(self, targets, _ctx=None):
            return _M_omero.model.Fileset._op_removeAllImageSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllImageSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeAllImageSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllImageSet(self, _r):
            return _M_omero.model.Fileset._op_removeAllImageSet.end(self, _r)

        def clearImages(self, _ctx=None):
            return _M_omero.model.Fileset._op_clearImages.invoke(self, ((), _ctx))

        def begin_clearImages(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_clearImages.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearImages(self, _r):
            return _M_omero.model.Fileset._op_clearImages.end(self, _r)

        def reloadImages(self, toCopy, _ctx=None):
            return _M_omero.model.Fileset._op_reloadImages.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadImages(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_reloadImages.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadImages(self, _r):
            return _M_omero.model.Fileset._op_reloadImages.end(self, _r)

        def unloadJobLinks(self, _ctx=None):
            return _M_omero.model.Fileset._op_unloadJobLinks.invoke(self, ((), _ctx))

        def begin_unloadJobLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_unloadJobLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadJobLinks(self, _r):
            return _M_omero.model.Fileset._op_unloadJobLinks.end(self, _r)

        def sizeOfJobLinks(self, _ctx=None):
            return _M_omero.model.Fileset._op_sizeOfJobLinks.invoke(self, ((), _ctx))

        def begin_sizeOfJobLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_sizeOfJobLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfJobLinks(self, _r):
            return _M_omero.model.Fileset._op_sizeOfJobLinks.end(self, _r)

        def copyJobLinks(self, _ctx=None):
            return _M_omero.model.Fileset._op_copyJobLinks.invoke(self, ((), _ctx))

        def begin_copyJobLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_copyJobLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyJobLinks(self, _r):
            return _M_omero.model.Fileset._op_copyJobLinks.end(self, _r)

        def addFilesetJobLink(self, target, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetJobLink.invoke(self, ((target, ), _ctx))

        def begin_addFilesetJobLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetJobLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFilesetJobLink(self, _r):
            return _M_omero.model.Fileset._op_addFilesetJobLink.end(self, _r)

        def addAllFilesetJobLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Fileset._op_addAllFilesetJobLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFilesetJobLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addAllFilesetJobLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFilesetJobLinkSet(self, _r):
            return _M_omero.model.Fileset._op_addAllFilesetJobLinkSet.end(self, _r)

        def removeFilesetJobLink(self, theTarget, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetJobLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFilesetJobLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetJobLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFilesetJobLink(self, _r):
            return _M_omero.model.Fileset._op_removeFilesetJobLink.end(self, _r)

        def removeAllFilesetJobLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Fileset._op_removeAllFilesetJobLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFilesetJobLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeAllFilesetJobLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFilesetJobLinkSet(self, _r):
            return _M_omero.model.Fileset._op_removeAllFilesetJobLinkSet.end(self, _r)

        def clearJobLinks(self, _ctx=None):
            return _M_omero.model.Fileset._op_clearJobLinks.invoke(self, ((), _ctx))

        def begin_clearJobLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_clearJobLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearJobLinks(self, _r):
            return _M_omero.model.Fileset._op_clearJobLinks.end(self, _r)

        def reloadJobLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Fileset._op_reloadJobLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadJobLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_reloadJobLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadJobLinks(self, _r):
            return _M_omero.model.Fileset._op_reloadJobLinks.end(self, _r)

        def getFilesetJobLink(self, index, _ctx=None):
            return _M_omero.model.Fileset._op_getFilesetJobLink.invoke(self, ((index, ), _ctx))

        def begin_getFilesetJobLink(self, index, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_getFilesetJobLink.begin(self, ((index, ), _response, _ex, _sent, _ctx))

        def end_getFilesetJobLink(self, _r):
            return _M_omero.model.Fileset._op_getFilesetJobLink.end(self, _r)

        def setFilesetJobLink(self, index, theElement, _ctx=None):
            return _M_omero.model.Fileset._op_setFilesetJobLink.invoke(self, ((index, theElement), _ctx))

        def begin_setFilesetJobLink(self, index, theElement, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_setFilesetJobLink.begin(self, ((index, theElement), _response, _ex, _sent, _ctx))

        def end_setFilesetJobLink(self, _r):
            return _M_omero.model.Fileset._op_setFilesetJobLink.end(self, _r)

        def getPrimaryFilesetJobLink(self, _ctx=None):
            return _M_omero.model.Fileset._op_getPrimaryFilesetJobLink.invoke(self, ((), _ctx))

        def begin_getPrimaryFilesetJobLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_getPrimaryFilesetJobLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPrimaryFilesetJobLink(self, _r):
            return _M_omero.model.Fileset._op_getPrimaryFilesetJobLink.end(self, _r)

        def setPrimaryFilesetJobLink(self, theElement, _ctx=None):
            return _M_omero.model.Fileset._op_setPrimaryFilesetJobLink.invoke(self, ((theElement, ), _ctx))

        def begin_setPrimaryFilesetJobLink(self, theElement, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_setPrimaryFilesetJobLink.begin(self, ((theElement, ), _response, _ex, _sent, _ctx))

        def end_setPrimaryFilesetJobLink(self, _r):
            return _M_omero.model.Fileset._op_setPrimaryFilesetJobLink.end(self, _r)

        def getJobLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Fileset._op_getJobLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getJobLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_getJobLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getJobLinksCountPerOwner(self, _r):
            return _M_omero.model.Fileset._op_getJobLinksCountPerOwner.end(self, _r)

        def linkJob(self, addition, _ctx=None):
            return _M_omero.model.Fileset._op_linkJob.invoke(self, ((addition, ), _ctx))

        def begin_linkJob(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_linkJob.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkJob(self, _r):
            return _M_omero.model.Fileset._op_linkJob.end(self, _r)

        def addFilesetJobLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetJobLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addFilesetJobLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetJobLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addFilesetJobLinkToBoth(self, _r):
            return _M_omero.model.Fileset._op_addFilesetJobLinkToBoth.end(self, _r)

        def findFilesetJobLink(self, removal, _ctx=None):
            return _M_omero.model.Fileset._op_findFilesetJobLink.invoke(self, ((removal, ), _ctx))

        def begin_findFilesetJobLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_findFilesetJobLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findFilesetJobLink(self, _r):
            return _M_omero.model.Fileset._op_findFilesetJobLink.end(self, _r)

        def unlinkJob(self, removal, _ctx=None):
            return _M_omero.model.Fileset._op_unlinkJob.invoke(self, ((removal, ), _ctx))

        def begin_unlinkJob(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_unlinkJob.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkJob(self, _r):
            return _M_omero.model.Fileset._op_unlinkJob.end(self, _r)

        def removeFilesetJobLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetJobLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeFilesetJobLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetJobLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeFilesetJobLinkFromBoth(self, _r):
            return _M_omero.model.Fileset._op_removeFilesetJobLinkFromBoth.end(self, _r)

        def linkedJobList(self, _ctx=None):
            return _M_omero.model.Fileset._op_linkedJobList.invoke(self, ((), _ctx))

        def begin_linkedJobList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_linkedJobList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedJobList(self, _r):
            return _M_omero.model.Fileset._op_linkedJobList.end(self, _r)

        def getTemplatePrefix(self, _ctx=None):
            return _M_omero.model.Fileset._op_getTemplatePrefix.invoke(self, ((), _ctx))

        def begin_getTemplatePrefix(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_getTemplatePrefix.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getTemplatePrefix(self, _r):
            return _M_omero.model.Fileset._op_getTemplatePrefix.end(self, _r)

        def setTemplatePrefix(self, theTemplatePrefix, _ctx=None):
            return _M_omero.model.Fileset._op_setTemplatePrefix.invoke(self, ((theTemplatePrefix, ), _ctx))

        def begin_setTemplatePrefix(self, theTemplatePrefix, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_setTemplatePrefix.begin(self, ((theTemplatePrefix, ), _response, _ex, _sent, _ctx))

        def end_setTemplatePrefix(self, _r):
            return _M_omero.model.Fileset._op_setTemplatePrefix.end(self, _r)

        def unloadAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Fileset._op_unloadAnnotationLinks.invoke(self, ((), _ctx))

        def begin_unloadAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_unloadAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadAnnotationLinks(self, _r):
            return _M_omero.model.Fileset._op_unloadAnnotationLinks.end(self, _r)

        def sizeOfAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Fileset._op_sizeOfAnnotationLinks.invoke(self, ((), _ctx))

        def begin_sizeOfAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_sizeOfAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfAnnotationLinks(self, _r):
            return _M_omero.model.Fileset._op_sizeOfAnnotationLinks.end(self, _r)

        def copyAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Fileset._op_copyAnnotationLinks.invoke(self, ((), _ctx))

        def begin_copyAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_copyAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyAnnotationLinks(self, _r):
            return _M_omero.model.Fileset._op_copyAnnotationLinks.end(self, _r)

        def addFilesetAnnotationLink(self, target, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetAnnotationLink.invoke(self, ((target, ), _ctx))

        def begin_addFilesetAnnotationLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetAnnotationLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFilesetAnnotationLink(self, _r):
            return _M_omero.model.Fileset._op_addFilesetAnnotationLink.end(self, _r)

        def addAllFilesetAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Fileset._op_addAllFilesetAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFilesetAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addAllFilesetAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFilesetAnnotationLinkSet(self, _r):
            return _M_omero.model.Fileset._op_addAllFilesetAnnotationLinkSet.end(self, _r)

        def removeFilesetAnnotationLink(self, theTarget, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetAnnotationLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFilesetAnnotationLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetAnnotationLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFilesetAnnotationLink(self, _r):
            return _M_omero.model.Fileset._op_removeFilesetAnnotationLink.end(self, _r)

        def removeAllFilesetAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Fileset._op_removeAllFilesetAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFilesetAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeAllFilesetAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFilesetAnnotationLinkSet(self, _r):
            return _M_omero.model.Fileset._op_removeAllFilesetAnnotationLinkSet.end(self, _r)

        def clearAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Fileset._op_clearAnnotationLinks.invoke(self, ((), _ctx))

        def begin_clearAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_clearAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearAnnotationLinks(self, _r):
            return _M_omero.model.Fileset._op_clearAnnotationLinks.end(self, _r)

        def reloadAnnotationLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Fileset._op_reloadAnnotationLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadAnnotationLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_reloadAnnotationLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadAnnotationLinks(self, _r):
            return _M_omero.model.Fileset._op_reloadAnnotationLinks.end(self, _r)

        def getAnnotationLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Fileset._op_getAnnotationLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getAnnotationLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_getAnnotationLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAnnotationLinksCountPerOwner(self, _r):
            return _M_omero.model.Fileset._op_getAnnotationLinksCountPerOwner.end(self, _r)

        def linkAnnotation(self, addition, _ctx=None):
            return _M_omero.model.Fileset._op_linkAnnotation.invoke(self, ((addition, ), _ctx))

        def begin_linkAnnotation(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_linkAnnotation.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkAnnotation(self, _r):
            return _M_omero.model.Fileset._op_linkAnnotation.end(self, _r)

        def addFilesetAnnotationLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetAnnotationLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addFilesetAnnotationLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_addFilesetAnnotationLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addFilesetAnnotationLinkToBoth(self, _r):
            return _M_omero.model.Fileset._op_addFilesetAnnotationLinkToBoth.end(self, _r)

        def findFilesetAnnotationLink(self, removal, _ctx=None):
            return _M_omero.model.Fileset._op_findFilesetAnnotationLink.invoke(self, ((removal, ), _ctx))

        def begin_findFilesetAnnotationLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_findFilesetAnnotationLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findFilesetAnnotationLink(self, _r):
            return _M_omero.model.Fileset._op_findFilesetAnnotationLink.end(self, _r)

        def unlinkAnnotation(self, removal, _ctx=None):
            return _M_omero.model.Fileset._op_unlinkAnnotation.invoke(self, ((removal, ), _ctx))

        def begin_unlinkAnnotation(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_unlinkAnnotation.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkAnnotation(self, _r):
            return _M_omero.model.Fileset._op_unlinkAnnotation.end(self, _r)

        def removeFilesetAnnotationLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetAnnotationLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeFilesetAnnotationLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_removeFilesetAnnotationLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeFilesetAnnotationLinkFromBoth(self, _r):
            return _M_omero.model.Fileset._op_removeFilesetAnnotationLinkFromBoth.end(self, _r)

        def linkedAnnotationList(self, _ctx=None):
            return _M_omero.model.Fileset._op_linkedAnnotationList.invoke(self, ((), _ctx))

        def begin_linkedAnnotationList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Fileset._op_linkedAnnotationList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedAnnotationList(self, _r):
            return _M_omero.model.Fileset._op_linkedAnnotationList.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.FilesetPrx.ice_checkedCast(proxy, '::omero::model::Fileset', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.FilesetPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Fileset'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_FilesetPrx = IcePy.defineProxy('::omero::model::Fileset', FilesetPrx)

    _M_omero.model._t_Fileset = IcePy.declareClass('::omero::model::Fileset')

    _M_omero.model._t_Fileset = IcePy.defineClass('::omero::model::Fileset', Fileset, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_usedFilesSeq', (), _M_omero.model._t_FilesetUsedFilesSeq, False, 0),
        ('_usedFilesLoaded', (), IcePy._t_bool, False, 0),
        ('_imagesSeq', (), _M_omero.model._t_FilesetImagesSeq, False, 0),
        ('_imagesLoaded', (), IcePy._t_bool, False, 0),
        ('_jobLinksSeq', (), _M_omero.model._t_FilesetJobLinksSeq, False, 0),
        ('_jobLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_jobLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_templatePrefix', (), _M_omero._t_RString, False, 0),
        ('_annotationLinksSeq', (), _M_omero.model._t_FilesetAnnotationLinksSeq, False, 0),
        ('_annotationLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0)
    ))
    Fileset._ice_type = _M_omero.model._t_Fileset

    Fileset._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Fileset._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Fileset._op_unloadUsedFiles = IcePy.Operation('unloadUsedFiles', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Fileset._op_sizeOfUsedFiles = IcePy.Operation('sizeOfUsedFiles', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Fileset._op_copyUsedFiles = IcePy.Operation('copyUsedFiles', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilesetUsedFilesSeq, False, 0), ())
    Fileset._op_addFilesetEntry = IcePy.Operation('addFilesetEntry', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetEntry, False, 0),), (), None, ())
    Fileset._op_addAllFilesetEntrySet = IcePy.Operation('addAllFilesetEntrySet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetUsedFilesSeq, False, 0),), (), None, ())
    Fileset._op_removeFilesetEntry = IcePy.Operation('removeFilesetEntry', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetEntry, False, 0),), (), None, ())
    Fileset._op_removeAllFilesetEntrySet = IcePy.Operation('removeAllFilesetEntrySet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetUsedFilesSeq, False, 0),), (), None, ())
    Fileset._op_clearUsedFiles = IcePy.Operation('clearUsedFiles', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Fileset._op_reloadUsedFiles = IcePy.Operation('reloadUsedFiles', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Fileset, False, 0),), (), None, ())
    Fileset._op_getFilesetEntry = IcePy.Operation('getFilesetEntry', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_int, False, 0),), (), ((), _M_omero.model._t_FilesetEntry, False, 0), ())
    Fileset._op_setFilesetEntry = IcePy.Operation('setFilesetEntry', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_int, False, 0), ((), _M_omero.model._t_FilesetEntry, False, 0)), (), ((), _M_omero.model._t_FilesetEntry, False, 0), ())
    Fileset._op_getPrimaryFilesetEntry = IcePy.Operation('getPrimaryFilesetEntry', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilesetEntry, False, 0), ())
    Fileset._op_setPrimaryFilesetEntry = IcePy.Operation('setPrimaryFilesetEntry', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetEntry, False, 0),), (), ((), _M_omero.model._t_FilesetEntry, False, 0), ())
    Fileset._op_unloadImages = IcePy.Operation('unloadImages', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Fileset._op_sizeOfImages = IcePy.Operation('sizeOfImages', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Fileset._op_copyImages = IcePy.Operation('copyImages', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilesetImagesSeq, False, 0), ())
    Fileset._op_addImage = IcePy.Operation('addImage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Image, False, 0),), (), None, ())
    Fileset._op_addAllImageSet = IcePy.Operation('addAllImageSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetImagesSeq, False, 0),), (), None, ())
    Fileset._op_removeImage = IcePy.Operation('removeImage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Image, False, 0),), (), None, ())
    Fileset._op_removeAllImageSet = IcePy.Operation('removeAllImageSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetImagesSeq, False, 0),), (), None, ())
    Fileset._op_clearImages = IcePy.Operation('clearImages', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Fileset._op_reloadImages = IcePy.Operation('reloadImages', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Fileset, False, 0),), (), None, ())
    Fileset._op_unloadJobLinks = IcePy.Operation('unloadJobLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Fileset._op_sizeOfJobLinks = IcePy.Operation('sizeOfJobLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Fileset._op_copyJobLinks = IcePy.Operation('copyJobLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilesetJobLinksSeq, False, 0), ())
    Fileset._op_addFilesetJobLink = IcePy.Operation('addFilesetJobLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetJobLink, False, 0),), (), None, ())
    Fileset._op_addAllFilesetJobLinkSet = IcePy.Operation('addAllFilesetJobLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetJobLinksSeq, False, 0),), (), None, ())
    Fileset._op_removeFilesetJobLink = IcePy.Operation('removeFilesetJobLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetJobLink, False, 0),), (), None, ())
    Fileset._op_removeAllFilesetJobLinkSet = IcePy.Operation('removeAllFilesetJobLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetJobLinksSeq, False, 0),), (), None, ())
    Fileset._op_clearJobLinks = IcePy.Operation('clearJobLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Fileset._op_reloadJobLinks = IcePy.Operation('reloadJobLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Fileset, False, 0),), (), None, ())
    Fileset._op_getFilesetJobLink = IcePy.Operation('getFilesetJobLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_int, False, 0),), (), ((), _M_omero.model._t_FilesetJobLink, False, 0), ())
    Fileset._op_setFilesetJobLink = IcePy.Operation('setFilesetJobLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_int, False, 0), ((), _M_omero.model._t_FilesetJobLink, False, 0)), (), ((), _M_omero.model._t_FilesetJobLink, False, 0), ())
    Fileset._op_getPrimaryFilesetJobLink = IcePy.Operation('getPrimaryFilesetJobLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilesetJobLink, False, 0), ())
    Fileset._op_setPrimaryFilesetJobLink = IcePy.Operation('setPrimaryFilesetJobLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetJobLink, False, 0),), (), ((), _M_omero.model._t_FilesetJobLink, False, 0), ())
    Fileset._op_getJobLinksCountPerOwner = IcePy.Operation('getJobLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Fileset._op_linkJob = IcePy.Operation('linkJob', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Job, False, 0),), (), ((), _M_omero.model._t_FilesetJobLink, False, 0), ())
    Fileset._op_addFilesetJobLinkToBoth = IcePy.Operation('addFilesetJobLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetJobLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Fileset._op_findFilesetJobLink = IcePy.Operation('findFilesetJobLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Job, False, 0),), (), ((), _M_omero.model._t_FilesetJobLinksSeq, False, 0), ())
    Fileset._op_unlinkJob = IcePy.Operation('unlinkJob', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Job, False, 0),), (), None, ())
    Fileset._op_removeFilesetJobLinkFromBoth = IcePy.Operation('removeFilesetJobLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetJobLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Fileset._op_linkedJobList = IcePy.Operation('linkedJobList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilesetLinkedJobSeq, False, 0), ())
    Fileset._op_getTemplatePrefix = IcePy.Operation('getTemplatePrefix', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Fileset._op_setTemplatePrefix = IcePy.Operation('setTemplatePrefix', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Fileset._op_unloadAnnotationLinks = IcePy.Operation('unloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Fileset._op_sizeOfAnnotationLinks = IcePy.Operation('sizeOfAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Fileset._op_copyAnnotationLinks = IcePy.Operation('copyAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilesetAnnotationLinksSeq, False, 0), ())
    Fileset._op_addFilesetAnnotationLink = IcePy.Operation('addFilesetAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetAnnotationLink, False, 0),), (), None, ())
    Fileset._op_addAllFilesetAnnotationLinkSet = IcePy.Operation('addAllFilesetAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetAnnotationLinksSeq, False, 0),), (), None, ())
    Fileset._op_removeFilesetAnnotationLink = IcePy.Operation('removeFilesetAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetAnnotationLink, False, 0),), (), None, ())
    Fileset._op_removeAllFilesetAnnotationLinkSet = IcePy.Operation('removeAllFilesetAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetAnnotationLinksSeq, False, 0),), (), None, ())
    Fileset._op_clearAnnotationLinks = IcePy.Operation('clearAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Fileset._op_reloadAnnotationLinks = IcePy.Operation('reloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Fileset, False, 0),), (), None, ())
    Fileset._op_getAnnotationLinksCountPerOwner = IcePy.Operation('getAnnotationLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Fileset._op_linkAnnotation = IcePy.Operation('linkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_FilesetAnnotationLink, False, 0), ())
    Fileset._op_addFilesetAnnotationLinkToBoth = IcePy.Operation('addFilesetAnnotationLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Fileset._op_findFilesetAnnotationLink = IcePy.Operation('findFilesetAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_FilesetAnnotationLinksSeq, False, 0), ())
    Fileset._op_unlinkAnnotation = IcePy.Operation('unlinkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, ())
    Fileset._op_removeFilesetAnnotationLinkFromBoth = IcePy.Operation('removeFilesetAnnotationLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilesetAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Fileset._op_linkedAnnotationList = IcePy.Operation('linkedAnnotationList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilesetLinkedAnnotationSeq, False, 0), ())

    _M_omero.model.Fileset = Fileset
    del Fileset

    _M_omero.model.FilesetPrx = FilesetPrx
    del FilesetPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero
